/*
 * notify.c
 * 
 * Copyright (C) 2005  Josef Kufner <jk@myserver.cz>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 * 
 */
#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include <stdio.h>

#include "options.h"
#include "notify.h"
#include "callbacks.h"
#include "support.h"

#define DEBUG 0
#include "debug.h"


/** Okno na ikonky */
static GtkWidget *notify_window = NULL;
static GtkWidget *main_hbox;

/* promenne na udrzovani pozice okna pri animacich */
static int target_x = -1, target_y = -1;
static int cur_x = -1, cur_y = -1;

/* info o mrtve ikone */
struct s_dead_icon {
	int height, width, just_started;
	GtkWidget *icon;
};

/* seznam mrtvych ikon */
static GList *dead_icon = NULL;
#define DI_DATA(p)  ((struct s_dead_icon *)((p)->data))


/* lokalni funkce */
static void notify_update_window (int with_animation);
static void notify_update_coords (void);


static gboolean notify_update_animation (gpointer data) {
	int run_again = 0;
	GList *pos, *tmp;
	
	for (pos = g_list_first(dead_icon); pos; pos = g_list_next(pos)) {
		if (DI_DATA(pos)->just_started) {
			DBG_MSG(8, "New dead icon detected -- updating it's data...");
			GtkRequisition size;
			gtk_widget_size_request(DI_DATA(pos)->icon, &size);
			DI_DATA(pos)->width = size.width;
			DI_DATA(pos)->height = size.height;
			DI_DATA(pos)->just_started = 0;
		}

		if (DI_DATA(pos)->height <= 0 && DI_DATA(pos)->width <= 0) {
			DBG_MSG(8, "Destroying dead icon...");
			gtk_widget_destroy(GTK_WIDGET(DI_DATA(pos)->icon));
			free(DI_DATA(pos));
			DBG_MSG(8, "Removing destroyed dead icon...");
			dead_icon = g_list_delete_link(dead_icon, pos);
			DBG_MSG(8, "Dead icon is dead.");
			break;
		} else {
			DBG_MSG(8, "Recalculating icon size...");
			gtk_widget_set_size_request(GTK_WIDGET(DI_DATA(pos)->icon),
					DI_DATA(pos)->width > 0 ? DI_DATA(pos)->width : 0,
					DI_DATA(pos)->height > 0 ? DI_DATA(pos)->height : 0);
			DI_DATA(pos)->width -= opt.anim_step;
			DI_DATA(pos)->height -= opt.anim_step;
			run_again = 1;
		}
	}
	
	notify_update_coords();

	if ((unsigned)(cur_x - target_x) < (unsigned)opt.anim_step) {
		cur_x = target_x;
	} else {
		cur_x += target_x > cur_x ? opt.anim_step : -opt.anim_step;
		run_again = 1;
	}

	if ((unsigned)(cur_y - target_y) < (unsigned)opt.anim_step) {
		cur_y = target_y;
	} else {
		cur_y += target_y > cur_y ? opt.anim_step : -opt.anim_step;
		run_again = 1;
	}

	gtk_window_move(GTK_WINDOW(notify_window), cur_x, cur_y);

	return run_again;
}

static void notify_start_animation (void) {
	g_timeout_add(opt.anim_delay, notify_update_animation, NULL);
}

static void notify_update_coords (void) {
	GdkScreen *screen;
	GdkWindow *win;
	gint height, width, wh, ww;
	GtkRequisition size;

	screen = gdk_screen_get_default();
	height = gdk_screen_get_height(screen);
	width = gdk_screen_get_width(screen);

	gtk_widget_size_request(notify_window, &size);
	target_x = width - size.width;
	target_y = height - size.height;
	
	gtk_window_get_position(GTK_WINDOW(notify_window), &cur_x, &cur_y);
}

static void notify_update_window (int with_animation) {

	notify_update_coords();
	
	if (with_animation) {
		if (cur_x != target_x || cur_y != target_y) {
			notify_start_animation();
		}
	} else {
		gtk_window_move(GTK_WINDOW(notify_window), target_x, target_y);
	}
}

/** Vytvoreni okna pro ikonky
 */
void notify_init(void) {
	GtkWidget *label;

	notify_window = gtk_window_new (GTK_WINDOW_POPUP);
	gtk_window_set_title (GTK_WINDOW (notify_window), _("GTK Notify"));
	gtk_window_set_resizable (GTK_WINDOW (notify_window), FALSE);
	gtk_window_set_icon_name (GTK_WINDOW (notify_window), "gtk-dialog-info");
	gtk_window_set_role (GTK_WINDOW (notify_window), "notify");
	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (notify_window), TRUE);
	gtk_window_set_skip_pager_hint (GTK_WINDOW (notify_window), TRUE);
	gtk_window_set_type_hint (GTK_WINDOW (notify_window), GDK_WINDOW_TYPE_HINT_DOCK);
	gtk_window_set_focus_on_map (GTK_WINDOW (notify_window), FALSE);
	gtk_window_set_keep_above(GTK_WINDOW(notify_window), TRUE);
	gtk_window_stick(GTK_WINDOW(notify_window));
	gtk_window_set_gravity (GTK_WINDOW(notify_window), GDK_GRAVITY_STATIC);

	main_hbox = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (main_hbox);
	gtk_container_add (GTK_CONTAINER (notify_window), main_hbox);

	/*
	label = gtk_label_new(">");
	gtk_widget_show (label);
	gtk_box_pack_start (GTK_BOX (main_hbox), label, TRUE, TRUE, 0);
	// */

	notify_update_window(0);
	gtk_widget_show_all (notify_window);
	
	return;
}

/** Uklid okna
 */
void notify_cleanup(void) {
	if (notify_window) {
		gtk_widget_destroy(notify_window);
		notify_window = NULL;
	}
}

/** Prida ikonku na konec
 */
GtkWidget *notify_append_icon(gchar *filename, gpointer data) {
	GtkWidget *button;
	GtkWidget *image;
	
	button = gtk_button_new ();
	gtk_widget_show (button);
	gtk_box_pack_start (GTK_BOX (main_hbox), button, TRUE, TRUE, 0);
	gtk_button_set_focus_on_click (GTK_BUTTON (button), FALSE);

	image = create_pixmap (notify_window, filename);
	gtk_widget_show (image);
	gtk_container_add (GTK_CONTAINER (button), image);

	g_signal_connect ((gpointer) button, "clicked",
        		G_CALLBACK (on_icon_button_clicked),
        		data);
	
	notify_update_window(opt.animations && opt.anim_show);
	
	return button;
}


/** Odebere ikonu ze zacatku
 */
void notify_remove_icon(GtkWidget *icon) {
	struct s_dead_icon *new_dead_icon;
	if (opt.animations && opt.anim_hide) {
		new_dead_icon = (struct s_dead_icon *)malloc(sizeof(struct s_dead_icon));
		new_dead_icon->icon = icon;
		new_dead_icon->just_started = 1;

		dead_icon = g_list_prepend(dead_icon, new_dead_icon);
		
		notify_start_animation();
	} else {
		gtk_widget_destroy(icon);
		notify_update_window(0);
	}
}

